package csci4534.controller;

/**
 * A disk block. Block data is accessible via their index (called the
 * <EM>data offset</EM>). Naturally, the data offset should be in the
 * range [0,<EM>c<SUB>b</SUB></EM>-1] where <EM>c<SUB>b</SUB></EM> is
 * the block size.
 *
 * @author Toli Lerios
 **/

public class Block
{

    // INSTANCE DATA.

    private byte[] mData;


    // CONSTRUCTORS.

    /**
     * Creates a new disk block. The new block's data is all zeroed
     * out.
     *
     * @param size The size <EM>c<SUB>b</SUB></EM> of the block.
     **/

    public Block(int size)
    {
        mData=new byte[size];
    }


    // PUBLIC INTERFACE.

    /**
     * Returns the block size <EM>c<SUB>b</SUB></EM>.
     *
     * @return The size.
     **/

    public int getSize()
    {
        return mData.length;
    }

    /**
     * Returns the block data at the given data offset.
     *
     * @param offset The data offset.
     *
     * @return The data.
     **/

    public byte getData(int offset)
    {
        return mData[offset];
    }

    /**
     * Sets the block data at the given data offset to the given
     * value.
     *
     * @param offset The data offset.
     * @param data The data value.
     **/

    public void setData(int offset,
                        byte data)
    {
        mData[offset]=data;
    }


    // Object.

    public String toString()
    {
        StringBuffer buffer=new StringBuffer();
        buffer.append('(');
        for (int i=0;i<getSize();i++) {
            if (i>0) {
                buffer.append(',');
            }
            buffer.append(getData(i));
        }
        buffer.append(')');
        return buffer.toString();
    }
}
