package csci4534.controller;

/**
 * An exception indicating that a controller cannot provide the
 * required actual capacity <EM>c<SUB>a</SUB></EM> given the total
 * capacity of its disks <EM>c<SUB>t</SUB></EM>. In other words, there
 * is insufficient disk space.
 *
 * @author Toli Lerios
 **/

public class InsufficientSpaceException
    extends Exception
{

    // INSTANCE DATA.

    private int mActualCapacity;
    private int mTotalCapacity;
    private String mConstraint;


    // CONSTRUCTORS.

    /**
     * Creates a new exception indicating insufficient disk space.
     *
     * @param actualCapacity The actual capacity
     * <EM>c<SUB>a</SUB></EM> of the controller.
     * @param totalCapacity The total capacity <EM>c<SUB>t</SUB></EM>
     * of the controller's disks.
     * @param constraint A textual description of the constraint
     * imposed on the actual and total capacities by the
     * controller. This text may refer to those capacities explicitly,
     * or express the constraint in terms of other controller
     * parameters.
     **/

    public InsufficientSpaceException(int actualCapacity,
                                      int totalCapacity,
                                      String constraint)
    {
        super("Constraint '"+constraint+"' not met by actual capacity "+
              actualCapacity+" and total capacity "+totalCapacity);
        mActualCapacity=actualCapacity;
        mTotalCapacity=totalCapacity;
        mConstraint=constraint;
    }


    // PUBLIC INTERFACE.

    /**
     * Returns the actual capacity <EM>c<SUB>a</SUB></EM> of the
     * controller.
     *
     * @return The capacity.
     **/

    public int getActualCapacity()
    {
        return mActualCapacity;
    }

    /**
     * Returns the total capacity <EM>c<SUB>t</SUB></EM> of the
     * controller's disks.
     *
     * @return The capacity.
     **/

    public int getTotalCapacity()
    {
        return mTotalCapacity;
    }

    /**
     * Returns a textual description of the constraint imposed on the
     * actual and total capacities by the controller.
     *
     * @return The constraint.
     **/

    public String getConstraint()
    {
        return mConstraint;
    }
}
