package csci4534.controller;

import java.util.StringTokenizer;

/**
 * A command-line disk controller simulator.
 *
 * @author Toli Lerios
 **/

public class MainSimulator
{

    // PUBLIC INTERFACE.

    /**
     * Main program.
     *
     * @param args The command-line arguments: execute without
     * arguments for usage instructions.
     *
     * @throws Exception Thrown iff an error occurs.
     **/

    public static void main(String[] args)
        throws Exception
    {

        // Error-checking.

        if (args.length<5) {
            System.err.println
                ("Usage: <controller> <disk count> <block count> <block size>"+
                 " <actual capacity>");
            System.err.println
                ("       ((r:<offset>)|(w:<offset>:<value>)|"+
                 "(d:<disk>(:<block>)))+");
            return;
        }

        // Parse fixed command-line arguments.

        Controller controller=(Controller)Class.forName(args[0]).newInstance();
        System.err.println("Controller: "+controller.getClass().getName());
        int diskCount=Integer.parseInt(args[1]);
        int blockCount=Integer.parseInt(args[2]);
        int blockSize=Integer.parseInt(args[3]);
        int actualCapacity=Integer.parseInt(args[4]);

        // Initialize disks, controller, and verifier.

        Disk[] disks=new Disk[diskCount];
        for (int i=0;i<diskCount;i++) {
            disks[i]=new Disk("D"+i,blockCount,blockSize);
        }
        controller.format(disks,actualCapacity);
        byte[] verifier=new byte[actualCapacity];

        // Run simulation.

        for (int i=5;i<args.length;i++) {
            StringTokenizer tokenizer=new StringTokenizer(args[i],":");
            String operation=tokenizer.nextToken();
            try {
                if (operation.equals("r")) {
                    int offset=Integer.parseInt(tokenizer.nextToken());
                    System.out.print("r@"+offset+"=");
                    byte value=controller.read(offset);
                    System.out.println(value);
                    if (verifier[offset]!=value) {
                        System.out.println
                            ("Read mismatch: expected "+verifier[offset]+
                             " but got "+value);
                    }
                } else if (operation.equals("w")) {
                    int offset=Integer.parseInt(tokenizer.nextToken());
                    byte value=Byte.parseByte(tokenizer.nextToken());
                    System.out.print("w@"+offset+"=");
                    controller.write(offset,value);
                    System.out.println(value);
                    verifier[offset]=value;
                } else if (operation.equals("d")) {
                    Disk disk=disks[Integer.parseInt(tokenizer.nextToken())];
                    if (!tokenizer.hasMoreTokens()) {
                        System.out.print("Damaging disk "+disk.getName()+": ");
                        disk.damage();
                        System.out.println("done");
                        continue;
                    }
                    int block=Integer.parseInt(tokenizer.nextToken());
                    System.out.print
                        ("Damaging disk "+disk.getName()+
                         ", block "+block+": ");
                    disk.damageBlock(block);
                    System.out.println("done");
                } else {
                    System.err.println("Skipping invalid operation: "+args[i]);
                }
            } catch (DiskException ex) {
                System.out.println(ex);
            }
        }

        // Display final disk state.

        for (int i=0;i<diskCount;i++) {
            System.out.println(disks[i]);
        }
    }
}
