package csci4534.controller;

// ADD CODE HERE IF APPROPRIATE.

/**
 * A generic RAID disk controller with parity blocks (with
 * block-interleaved, and possibly distributed, parity). There are no
 * spares, but there is redundancy in the form of block parity. If a
 * single block on a stripe becomes inaccessible, its failure is never
 * reported to the controller user. Instead:
 *
 * <UL>
 *
 * <LI> Read operations reconstruct a missing data block from the
 * remaining blocks of the same stripe. If the parity block is the one
 * that got damaged, read operations are not affected at all since
 * they do not need the parity in this case.
 *
 * <P>
 *
 * <LI> Simiarly, write operations don't store the byte in the block
 * itself (since it's inaccessible), but instead update the parity
 * block so that the actual byte value can be reconstructed during a
 * read; if it was the parity block that got damaged, we simply don't
 * perform parity computations.
 *
 * </UL>
 *
 * Note that, due to striping, the above scenario still applies in the
 * case of a complete disk failure: as long as all other disks are
 * good and their blocks accessible, each stripe would have only lost
 * one block due to the damaged disk.
 *
 * <P>
 *
 * Parity updates are efficient: only two disk byte reads and two
 * writes are needed for each controller byte write as long as all
 * blocks of a stripe are accessible. If a single <EM>data</EM> block
 * fails (due to a disk failure or a single block becoming
 * inaccessible), parity updates require reading all blocks of the
 * stripe. Finally, if a second block of the same stripe gets damaged,
 * then operations on the inaccessible blocks always directly result
 * into permanent controller failures (i.e. byte requests with offsets
 * mapped to the damaged areas will never succeed again); however,
 * operations on the accessible blocks continue to succeed.
 *
 * <P>
 *
 * Parity calculations can get complicated if some disk blocks are not
 * used by the controller. As long as they remain accessible,
 * incorporating those blocks in parity calculations is simply
 * wasteful: as they contain only zeros, they do not impact the result
 * of the computation. However, as soon as such unused blocks become
 * inaccessible, they are problematic. For example, if a stripe has
 * one inaccessible data block and one inaccessible unused block,
 * parity computations fail; while, in fact, they should succeed
 * because the unused block should be altogether ignored. As a result,
 * and in order to keep parity calculations uniform and fast in
 * hardware, parity controllers require that the actual capacity
 * <EM>c<SUB>a</SUB></EM> is always equal to the total capacity
 * <EM>c<SUB>t</SUB></EM>.
 *
 * @author Toli Lerios
 **/

abstract public class ParityController
    extends ControllerBase
{

    // INSTANCE DATA.

    // ADD CODE HERE IF APPROPRIATE.


    // SUBCLASS INTERFACE.

    /**
     * Returns the location information of a byte under the
     * controller.
     *
     * @param offset The controller's byte offset.
     **/

    protected abstract ParityBytePosition getBytePosition(int offset);


    // ControllerBase.

    public void format(Disk[] disks,
                       int actualCapacity)
        throws InsufficientSpaceException
    {

        // ADD CODE HERE IF APPROPRIATE.

    }

    public byte read(int offset)
        throws DiskException
    {

        // ADD CODE HERE IF APPROPRIATE.

        // REMOVE THE LINE BELOW.
        return 0;
    }

    public void write(int offset,
                      byte data)
        throws DiskException
    {

        // ADD CODE HERE IF APPROPRIATE.

    }
}
