package csci4534.controller;

// ADD CODE HERE IF APPROPRIATE.

/**
 * A RAID 1 disk controller (with disk mirroring and spare disk
 * support). Bytes are stored in order on the blocks of the first
 * disk, and then on the blocks of the second disk, and so on. No
 * spare blocks are set aside on each disk. However, each disk is
 * mirrored at all times by another disk, and if there are still
 * unused disks, they are set aside as spares. Thus, if a block
 * becomes inaccessible on the primary disk (or the primary disk is no
 * longer good), then we generate a controller failure only if the
 * block on the mirror disk is also inaccessible (or the mirror disk
 * is no longer good). Whenever any damage (even a single inaccessible
 * block) is detected on either the primary disk or its mirror, and if
 * spares are available, one (or both) of the damaged disks is (are)
 * completely replaced; our assumption is that, once a disk is old
 * enough to have one inaccessible block, its failure is imminent and
 * it's best to remove it altogether.
 *
 * <P>
 *
 * When we build a new mirror on a spare, we do a raw complete disk
 * copy of all block data (whether those blocks contain controller
 * bytes or not) from either the primary or mirror disk (whoever has
 * that block's data still accessible; if neither, we just make sure
 * the spare's block is accessible and just leave it zeroed out); this
 * ensures that the spare is in good shape, without <EM>any</EM>
 * inaccessible blocks, and thus likely to have a reasonably long
 * remaining lifespan. So spares that are faulty in the least way are
 * completely discarded, and we proceed to build a new mirror on
 * another spare disk. If we successfully build a mirror on a perfect
 * disk, it becomes the primary disk. The new mirror will be either
 * the old primary or the old mirror: we choose the one that showed
 * fewer inaccessible blocks during the raw copy. Of course, if there
 * are still more spares available, and we know the new mirror has
 * inaccessible blocks, we immediately replace it with yet another
 * mirror we build on a yet another spare.
 *
 * <P>
 *
 * Once we run out of spares, and a block and its mirror both become
 * inaccessible, then controller failures become permanent (i.e. byte
 * requests with offsets mapped to the damaged areas will never
 * succeed again). But as long as at least one new mirror can be built
 * on a spare, failures are transitory: the damaged disk(s) is (are)
 * replaced with good one(s), and so the offset that caused the
 * earlier failure will no longer cause a failure during later
 * operations.
 *
 * @author Toli Lerios
 **/

public class RAID1Controller
    extends ControllerBase
{

    // INSTANCE DATA.

    // ADD CODE HERE IF APPROPRIATE.


    // PRIVATE HELPERS.

    /**
     * Remaps the given disk to a new index, among the array of the
     * controller's disks. The disk at the new index is also moved to
     * the original index, therefore this operation is actually a
     * swap.
     *
     * @param oldIndex The original disk index.
     * @param newIndex The new disk index.
     **/

    private void remapDisk(int oldIndex,
                           int newIndex)
    {

        // ADD CODE HERE IF APPROPRIATE.

    }

    /**
     * Attempts to build a new mirror disk on a spare disk, for the
     * primary disk with the given index. If a perfect spare is
     * successfully built, it replaces the primary disk under its disk
     * index. Also, the least defective of the old primary and old
     * mirror disks becomes the new mirror. If the new mirror is
     * damaged and there are more spares available, we build yet
     * another mirror.
     *
     * @param diskIndex The primary disk index.
     **/

    private void buildMirror(int diskIndex)
    {

        // ADD CODE HERE IF APPROPRIATE.

    }


    // ControllerBase.

    public void format(Disk[] disks,
                       int actualCapacity)
        throws InsufficientSpaceException
    {

        // ADD CODE HERE IF APPROPRIATE.

    }

    public byte read(int offset)
        throws DiskException
    {

        // ADD CODE HERE IF APPROPRIATE.

        // REMOVE THE LINE BELOW.
        return 0;
    }

    public void write(int offset,
                      byte data)
        throws DiskException
    {

        // ADD CODE HERE IF APPROPRIATE.

    }
}
