package csci4534.controller;

/**
 * A simple disk controller. Bytes are stored in order on the blocks
 * of the first disk, and then on the blocks of the second disk, and
 * so on. There are neither spares nor redundant storage; if the
 * actual capacity is smaller than the total capacity, the extra bytes
 * are never used. If a block becomes inaccessible or a disk gets
 * damaged, there is no way to recover the lost data, and there is no
 * corrective action. As a result, such disk failures always directly
 * result into permanent controller failures (i.e. byte requests with
 * offsets mapped to the damaged areas will never succeed again).
 * 
 * @author Toli Lerios
 **/

public class SimpleController
    extends ControllerBase
{

    // ControllerBase.

    public void format(Disk[] disks,
                       int actualCapacity)
        throws InsufficientSpaceException
    {
        super.format(disks,actualCapacity);
        if (actualCapacity>getTotalCapacity()) {
            throwInsufficientSpace("Actual<=Total");
        }
    }

    public byte read(int offset)
        throws DiskException
    {
        return (new SimpleBytePosition(offset,getBlockCount(),getDisks())).
            getData();
    }

    public void write(int offset,
                      byte data)
        throws DiskException
    {
        (new SimpleBytePosition(offset,getBlockCount(),getDisks())).
            setData(data);
    }
}
